#!/usr/bin/awk -f
#! -*- coding:utf-8; mode:awk; -*-


BEGIN {
    FS = ":";
    docstring = 0;  # Whether the current line is a DocString.
    classify = "";  # Classification of the current line.
    #file = "...";  # Current module source file name; to be set by the caller.
    printf("<pre>\n");
}

END {
    printf("</pre>\n");
}

/.*/ {
    classify = "";
    if ($1 ~ /^\s*-\s*$/) {
        # non-executable line
        if ($3 ~ /^\s*\/\*\*/) { docstring = 1; }
        if ($2 == 0) {
            classify = "meta";
        } else if (docstring) {
            classify = "docstring";
        } else {
            classify = "other";
        }
        if ($3 ~ /\*\/\s*$/) { docstring = 0; }
    } else if ($1 ~ /^\s*[1-9][0-9]*\s*$/) {
        classify = "exec";
    } else if ($1 ~ /^\s*#+\s*$/) {
        classify = "noexec";
    } else if ($1 ~ /^\s*=+\s*$/) {
        classify = "noexec-except";
    } else {
        classify = "unknown";
        printf("warning: could not classify line: %s\n", $0) | "cat 1>&2";
    }
}

/.*/ {
    lineno = 0 + $2;
    label = sprintf("%s:%d", file, lineno);
    code = $0;
    execcnt = -1;
    title = "";
    if (classify == "exec") {
        execcnt = 0 + $1;
    } else if (classify ~ /^noexec/) {
        execcnt = 0;
    }
    if (execcnt > 0) {
        title = sprintf("This line was executed %d times.", execcnt);
    } else if (execcnt == 0) {
        title = "This line was never executed.";
    } else {
        title = "This line is not subject to coverage measurement.";
    }
    sub(/^[^:]*:[^:]*:/, "", code);
    gsub(/[&]/, "\\&amp;", code);
    gsub(/[<]/, "\\&lt;", code);
    gsub(/[>]/, "\\&gt;", code);
    gsub(/["]/, "\\&quot;", code);
    gsub(/[']/, "\\&apos;", code);
    # These two have nothing to do with coverage testing but I like having them
    # stand out in the generated report nontheless.
    gsub(/TODO:/, "<span class=\"todo\">TODO:</span>", code);
    gsub(/FIXME:/, "<span class=\"fixme\">FIXME:</span>", code);
    printf("<span class=\"lineno\">");
    if (lineno > 0) {
        printf("<a id=\"%s\" href=\"#%s\">%6d</a>", label, label, lineno);
    } else {
        printf("%6d", lineno);
    }
    printf("</span>");
    printf("<span class=\"%s\" title=\"%s\">%s</span>\n",
           classify, title, code);
}
