/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file global.hpp
 *
 * @brief
 *         Global package configuration for our own source code files.
 *
 * Every implementation (`*.cpp`) file should `#include` this header file
 * before doing anything else.  It will `#include "config.h"` (if available) so
 * it need not be included again.
 *
 */

#ifndef BSC_KLAMMLER_GLOBAL_HPP
#define BSC_KLAMMLER_GLOBAL_HPP

#if HAVE_CONFIG_H
#  include <config.h>
#else
#  warning "Missing package configuration header 'config.h'"
#endif

#if HAVE_VALGRIND_VALGRIND_H
#  include <valgrind/valgrind.h>
#else
#  define RUNNING_ON_VALGRIND 0
#endif

#if HAVE_VAR_ATTRIBUTE_UNUSED
#  define BSC_KLAMMLER_ATTRIBUTE_UNUSED __attribute__((unused))
#endif

#if HAVE_FUNC_ATTRIBUTE_DEPRECATED
#  define BSC_KLAMMLER_ATTRIBUTE_DEPRECATED __attribute__((deprecated))
#endif

#if HAVE_FUNC_ATTRIBUTE_WARNING
#  define BSC_KLAMMLER_ATTRIBUTE_WARNING(M) __attribute__((warning(M)))
#endif

// This is correct.  'config.h' contains a '#define restrict __restrict__' (or
// similar) that makes 'restrict' look like a C++ keyword.
#define BSC_KLAMMLER_RESTRICT restrict

#ifndef BSC_KLAMMLER_FLOATING_POINT_TYPE
#  define BSC_KLAMMLER_FLOATING_POINT_TYPE double
#endif

// Include 'macros.hpp' to provide safe defaults for macros that we did not
// '#define'.
#include "macros.hpp"

#if BSC_KLAMMLER_WITH_BOOST_GIL && BSC_KLAMMLER_BOOST_GIL_BUG_8
#  define int_p_NULL static_cast<int*>(nullptr)
#  define png_bytep_NULL static_cast<png_bytep>(nullptr)
#  define png_infopp_NULL static_cast<png_infopp>(nullptr)
#endif

/**
 * @brief
 *         Default namespace for this project.
 *
 */
namespace bsc_klammler { /* only for Doxygen */ }

// TODO: Remove this crap!
extern "C" inline void set_break_point_here() {}

#endif  // ifndef BSC_KLAMMLER_GLOBAL_HPP
