dnl -*- coding:utf-8; mode:m4; -*-

dnl SYNOPSYS
dnl
dnl   AX_IS_SVN_CHECKOUT
dnl
dnl DESCRIPTION
dnl
dnl   Determine whether the current working copy is a Subversion (SVN) checkout
dnl   or an exported version, perhaps an extracted tarball.  Set the variable
dnl   ax_is_svn_checkout to 0 if the package does not appear to be a SVN
dnl   checkout and to an integer greater than zero otherwise.
dnl
dnl LICENSE
dnl
dnl   Copyright (c) 2013 Moritz Klammler <moritz@klammler.eu>
dnl
dnl   Copying and distribution of this file, with or without modification, are
dnl   permitted in any medium without royalty provided the copyright notice and
dnl   this notice are preserved.  This file is offered as-is, without any
dnl   warranty.
dnl
AC_DEFUN([AX_IS_SVN_CHECKOUT], [
  ax_is_svn_checkout=0
  # Assume that it can't be a SVN checkout if the svnversion program isn't
  # even installed.  This makes sense since in this case, any recipes
  # intended to be executed in a SVN checkout aren't likely to work eiter.
  AC_CHECK_PROGS([_ax_svnversion], [svnversion])
  AC_MSG_CHECKING([whether the project is a Subversion checkout])
  AS_IF([test "x$_ax_svnversion" = x],
        [AC_MSG_RESULT([no])],
        [
          AS_IF(
            [test "$($_ax_svnversion)" = "exported"],
            [AC_MSG_RESULT([no])],
            [test "$($_ax_svnversion)" = "Unversioned directory"],
            [AC_MSG_RESULT([no])],
            [
              AC_MSG_RESULT([yes])
              ax_is_svn_checkout=1
            ])
        ])
  unset _ax_svnversion
])
