dnl -*- coding:utf-8; mode:m4; -*-

dnl SYNOPSYS
dnl
dnl   MY_BOOST_TEST(LIBS)
dnl
dnl DESCRIPTION
dnl
dnl   Checks how Boost.Test can be used.  First, try linking to the precompiled
dnl   unit test framework library LIBS dynamically or statically.  If the user
dnl   provides a value via '--with-boost-test=LIBS' then that value is used
dnl   instead.  If linking fails, try using the pure-header version of the
dnl   library.
dnl
dnl   The shell variable 'my_boost_test_type' is set to a value of
dnl   'LINK_DYNAMIC', 'LINK_STATIC', 'INCLUDE' or the empty string depending on
dnl   whether Boost.Test can be linked in dynamically, statically, used in its
dnl   pure-header version or cannot be used at all.  If the user requested
dnl   Boost.Test not to be used, it will be set to the empty string.  If
dnl   Boost.Test can be used, AC_DEFINE is called on HAVE_BOOST_TEST.  If the
dnl   dynamic link version can be used, AC_DEFINE is also called on
dnl   BOOST_TEST_DYN_LINK.  Finally, AC_SUBST is called for BOOST_TEST_LIBS to
dnl   contain the libraries required to link to the Boost Unit Test Framework
dnl   (possibly none).
dnl
dnl LICENSE
dnl
dnl   Copyright (c) 2014 Moritz Klammler <moritz.klammler@student.kit.edu>
dnl
dnl   Copying and distribution of this file, with or without modification, are
dnl   permitted in any medium without royalty provided the copyright notice and
dnl   this notice are preserved.  This file is offered as-is, without any
dnl   warranty.
dnl
AC_DEFUN([MY_BOOST_TEST],
[
  BOOST_TEST_LIBS='$1'
  my_boost_test_type=
  AC_ARG_WITH(
    [boost-test],
    [AS_HELP_STRING(
      [--with-boost-test[=LIBS]],
      [
        Boost Unit Test Framework (UTF) to use.  Libraries required to link to
        the UTF can be provided in the optional LIBS argument (such as '$1',
        which is the default).  Alternatively, the constant 'yes' switches back
        to the default and 'no' disables using the Boost UTF.)
      ])
    ],
    [
      AS_CASE(
        [$withval],
        [yes],
        [
          my_boost_test_type=
          BOOST_TEST_LIBS='$1'
        ],
        [no],
        [
          my_boost_test_type=
          BOOST_TEST_LIBS=
        ],
        [
          my_boost_test_type=
          BOOST_TEST_LIBS="$withval"
        ])
    ])
  AC_MSG_CHECKING([how Boost.Test can be used])
  AC_LANG_PUSH([C++])
  # ---------------- #
  #   LINK_DYNAMIC   #
  # ---------------- #
  AS_IF(
    [test "x$my_boost_test_type" = x],
    [
      my_saved_libs="$LIBS"
      LIBS="$BOOST_TEST_LIBS $LIBS"
      AC_LINK_IFELSE(
        [AC_LANG_SOURCE([
#define BOOST_TEST_DYN_LINK 1
#define BOOST_TEST_MODULE Test
#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_CASE(test1) { BOOST_REQUIRE(1); }
        ])],
        [
          my_boost_test_type=LINK_DYNAMIC
          AC_DEFINE([BOOST_TEST_DYN_LINK],
                    [1],
                    [Define to 1 if Boost.Test is to be linked dynamically.])
        ])
        LIBS="$my_saved_libs"
        unset my_saved_libs
    ])
  # --------------- #
  #   LINK_STATIC   #
  # --------------- #
  AS_IF(
    [test "x$my_boost_test_type" = x],
    [
      my_saved_libs="$LIBS"
      LIBS="$BOOST_TEST_LIBS $LIBS"
      AC_LINK_IFELSE(
        [AC_LANG_SOURCE([
#define BOOST_TEST_MODULE Test
#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_CASE(test1) { BOOST_REQUIRE(1); }
        ])],
        [
          my_boost_test_type=LINK_STATIC
        ])
        LIBS="$my_saved_libs"
        unset my_saved_libs
    ])
  # ----------- #
  #   INCLUDE   #
  # ----------- #
  AS_IF(
    [test "x$my_boost_test_type" = x],
    [
      AC_LINK_IFELSE(
        [AC_LANG_SOURCE([
#define BOOST_TEST_MODULE Test
#include <boost/test/included/unit_test.hpp>

BOOST_AUTO_TEST_CASE(test1) { BOOST_REQUIRE(1); }
        ])],
        [
          my_boost_test_type=INCLUDE
        ])
    ])
  AC_LANG_POP([C++])
  AC_DEFINE([HAVE_BOOST_TEST_LINK_DYNAMIC],
            [1],
            [Link Boost.Test dynamically (possible value of HAVE_BOOST_TEST).])
  AC_DEFINE([HAVE_BOOST_TEST_LINK_STATIC],
            [2],
            [Link Boost.Test statically (possible value of HAVE_BOOST_TEST).])
  AC_DEFINE([HAVE_BOOST_TEST_INCLUDE],
            [3],
            [Use pure-header version of Boost.Test (possible value of HAVE_BOOST_TEST).])
  AC_MSG_RESULT([$my_boost_test_type])
  AS_IF(
    [test -n "$my_boost_test_type"],
    [
      AC_DEFINE_UNQUOTED([HAVE_BOOST_TEST],
                [HAVE_BOOST_TEST_$my_boost_test_type],
                [Define to the (preferred) way Boost.Test can be used.])
    ])
  AC_SUBST([BOOST_TEST_LIBS], ["$BOOST_TEST_LIBS"])
])
