dnl -*- coding:utf-8; mode:m4; -*-

dnl SYNOPSYS
dnl
dnl   MY_CXX_EIGEN
dnl
dnl DESCRIPTION
dnl
dnl   Checks whether the Eigen3 linear algebra C++ librarx can be used.  Sets
dnl   the variable 'my_have_eigen' to 'yes' or 'no' and calls AC_DEFINE for
dnl   HAVE_EIGEN.  The variable 'my_eigen_version' is set to the version number
dnl   of the found Eigen library or to the empty string, if Eigen is not found.
dnl
dnl
dnl LICENSE
dnl
dnl   Copyright (c) 2014 Moritz Klammler <moritz.klammler@student.kit.edu>
dnl
dnl   Copying and distribution of this file, with or without modification, are
dnl   permitted in any medium without royalty provided the copyright notice and
dnl   this notice are preserved.  This file is offered as-is, without any
dnl   warranty.
dnl
AC_DEFUN([MY_CXX_EIGEN],
[
  my_have_eigen=
  my_eigen_version=
  AC_REQUIRE([AC_PROG_GREP])
  AC_REQUIRE([AC_PROG_SED])
  AC_MSG_CHECKING([for Eigen])
  AC_LANG_PUSH([C++])
  AC_LINK_IFELSE(
    [AC_LANG_SOURCE([[
#ifdef __GNUC__
#  pragma GCC diagnostic ignored "-Wunused-local-typedefs"
#endif
#include <Eigen/Dense>

int main()
{
  Eigen::Matrix<double, 3, 2> A;
  Eigen::Matrix<double, 2, 3> B;
  Eigen::Matrix<double, 3, 3> C;
  A <<
    11.0, 12.0,
    21.0, 22.0,
    31.0, 32.0;
  B <<
    11.0, 12.0, 13.0,
    21.0, 22.0, 23.0;
  C = A * B;
  return 0;
}
    ]])],
    [
      AC_PREPROC_IFELSE([AC_LANG_SOURCE([[
#include <Eigen/Core>
#define MY_EIGEN_VERSION_ARRAY                                                \
  {EIGEN_WORLD_VERSION, EIGEN_MAJOR_VERSION, EIGEN_MINOR_VERSION}
static const int eigen_version[] = MY_EIGEN_VERSION_ARRAY;
        ]])],
        [
          my_eigen_version="$(                                                \
            $GREP 'static const int eigen_version' conftest.i                 \
              | $SED -e 's/.*[{]//g' -e 's/[}].*//g' -e 's/ //g' -e 's/,/./g' \
            )"
        ],
        [AC_MSG_FAILURE([unexpected preprocessor failure])])
      my_have_eigen=yes
      AC_MSG_RESULT([$my_eigen_version])
      AC_DEFINE([HAVE_EIGEN],
                [1],
                [Define if you have Eigen])
    ],
    [
      my_have_eigen=no
      AC_MSG_RESULT([no])
    ])
  AC_LANG_POP([C++])
])
