#!/usr/bin/automake
#! -*- coding:utf-8; mode:makefile-automake; -*-


## Copyright (c) 2014 Moritz Klammler
##
## Permission is  hereby granted, free  of charge,  to any person  obtaining a
## copy of this software and  associated documentation files (the "Software"),
## to deal in  the Software without restriction,  including without limitation
## the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
## and/or  sell copies  of the  Software, and  to permit  persons to  whom the
## Software is furnished to do so, subject to the following conditions:
##
## The above copyright notice and this  permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
## FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
## THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
## FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
## DEALINGS IN THE SOFTWARE.


# Include this file into every 'Makefile.am' if and only if this is a
# Subversion checkout.  Use the Automake conditional 'IS_SVN_CHECKOUT' to
# decide whether it is.

EXTRA_DIST += .revision

update:
	-svn update
	${AM_V_at}${sh_init};                                                 \
	for f in `find . -name '.svnignore'`; do                              \
	  ${gen_printf} 'SORT' "$$f";                                         \
	  sort -u "$$f" > "$$f.sorted" && mv "$$f.sorted" "$$f" || ${fail};   \
	  ${gen_printf} 'SVN' "$$f";                                          \
	  svn propset 'svn:ignore' -F "$$f" "$${f%.svnignore}";               \
	done

commit: update log.txt
	${AM_V_at}${have_internet_access}
	${AM_V_at}${MAKE} ${AM_MAKEFLAGS} ChangeLog
	${AM_V_at}${MAKE} ${AM_MAKEFLAGS} fix-space
	${AM_V_at}${MAKE} ${AM_MAKEFLAGS} distcheck
	svn commit -F log.txt
	${AM_V_at}rm -f log.txt log.txt~
	${AM_V_at}${MAKE} ${AM_MAKEFLAGS} update

log.txt: FORCE
	@if test "x${EDITOR}" = x;                                            \
	then                                                                  \
	  echo "error: please set the EDITOR environment / make variable to"  \
	    "the text editor of your choice" >&2;                             \
	  ${fail};                                                            \
	fi
	${AM_V_at}${EDITOR} $@
	@if ${GREP} '[a-zA-Z0-9]' $@; then ${noop};                           \
	else                                                                  \
	  echo "aborting due to empty commit message" >&2;                    \
	  rm -f $@;                                                           \
	  ${fail};                                                            \
	fi

.revision: FORCE update
	@${gen_printf} 'GEN' '$@'
	${AM_V_at}expr $$(svnversion . | ${GREP} -o '[0-9]\+') + 1 > $@

fix-space:
	${AM_V_at}${sh_init};                                                 \
	test "${builddir}" = "${srcdir}" || ${fail};                          \
	temp=;                                                                \
	cleanup () {                                                          \
	  test "x$${temp}" = x || rm -f "$${temp}";                           \
	};                                                                    \
	trap cleanup EXIT || ${fail};                                         \
	temp="$$(mktemp .fix-space.XXXXXX)" || ${fail};                       \
	raw_list_files () {                                                   \
	  if ${have_internet_access}; then svn list -R $$@; fi;               \
	  svn status $$@ | ${GREP} '^[AM]' | ${SED} -e 's,^.\{7\},,g';        \
	};                                                                    \
	for file in `raw_list_files ${srcdir} | sort -u`;                     \
	do                                                                    \
	  if test -f "$${file}";                                              \
	  then                                                                \
	    if test -L "$${file}"; then continue; fi;                         \
	    istext=0;                                                         \
	    case `svn propget 'svn:mime-type' "$${file}"` in                  \
	      '')      istext=1; ;;                                           \
	      text/*)  istext=1; ;;                                           \
	    esac;                                                             \
	    if test $${istext} -gt 0;                                         \
	    then                                                              \
	      ${gen_printf} 'FIX' "$${file}";                                 \
	      cp "$${file}" "$${file}${backup_suffix}"                        \
	        && ${AWK} -f "${srcdir}/${auxdir}/fix-space.awk"              \
	           "$${file}" > "$${temp}"                                    \
	        && cat "$${temp}" > "$${file}"                                \
	        || ${fail};                                                   \
	    fi                                                                \
	  fi                                                                  \
	done

PHONIES += FORCE commit fix-space update
INTERMEDIATES += .revision
