/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


#ifndef BSC_KLAMMLER_NEST_HPP
#define BSC_KLAMMLER_NEST_HPP

#include <cstddef>  // std::size_t

namespace bsc_klammler
{
  namespace nest
  {
    /**
     * @brief
     *         Inverts a square matrix of `double`s using the Newton-Strassen
     *         (NeSt) algorithm by Sanders et al.
     *
     * Matrix data is passed in and out as flattened data in column-major order
     * via the arrays.  It is allowed to pass the same pointer twice in which
     * case the original matrix will be overwritten by its inverse.
     *
     * @param data_in
     *         elements of the matrix to invert
     *
     * @param data_out
     *         storage for the elements of the inverted matrix
     *
     * @param size
     *         size of the matrix (and its inverse)
     *
     * @param spd
     *         whether the matrix is symmetric and positive definite
     *
     */
    void
    invert(const double *const in,
           double *const out,
           const std::size_t size,
           const bool spd);
  }  // namespace nest
}  // namespace bsc_klammler

#endif  // ifndef BSC_KLAMMLER_NEST_HPP
