/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file module.hpp
 *
 * @brief
 *         Convenience header to ease the use of Boost.Test.
 *
 * Example usage for defining a test module `Foo`:
 *
 *     // Always include first if there is no good reason not to.  DEFINITELY
 *     // include it before this header file.
 *     #include "../global.hpp"
 *
 *     // Include header for the module to test.
 *     #include "../foo.hpp"
 *
 *     // Include any other headers that are needed for the test.
 *     #include <cstring>
 *
 *     // Define to the name of the test module BEFORE including this header.
 *     #define BOOST_TEST_MODULE Foo
 *     #include "boost_test_module.hpp"
 *
 *     BOOST_AUTO_TEST_CASE(something)
 *     {
 *       const char *const text = "lorem ipsum";
 *       const foo::Foo o {text};
 *       const std::size_t expected {std::strlen(text)};
 *       const std::size_t actual {o.getTextLength()};
 *       BOOST_CHECK_EQUALS(expected, actual);
 *     }
 *
 *     BOOST_AUTO_TEST_CASE(something_else)
 *     {
 *       // test something else...
 *     }
 *
 * This header file must be included *exactly* once in each test module.  It is
 * not possible to implement an other number than one test module in one
 * translation unit.  This file `#include`s `boost/test/unit_test.hpp` (or
 * `boost/test/included/unit_test.hpp` if dynamic linkage is not desired) but
 * defines no user macros itself.
 *
 * The `boost/test/floating_point_comparison.hpp` header is also included.
 *
 */

#ifndef BSC_KLAMMLER_TEST_MODULE_HPP
#  define BSC_KLAMMLER_TEST_MODULE_HPP
#else
#  error "'test/boost_test_module.hpp' included multiple times"
#endif


#ifdef HAVE_BOOST_TEST
#  if HAVE_BOOST_TEST == HAVE_BOOST_TEST_INCLUDE
// This loads the pure-header version of the library but slows down compilation
// significantly.
#    include <boost/test/included/unit_test.hpp>
#  else
// This requires linking against libboost_unit_test_framework but compiles much
// faster.
#    include <boost/test/unit_test.hpp>
#  endif
#  include <boost/test/floating_point_comparison.hpp>
#else
#  error "The Boost Unit Test Framework is required"
#endif
