/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file test_control_flow_error.cpp
 *
 * @brief
 *         Unit tests for the `control_flow_error.hpp` header.
 *
 */

#include "../global.hpp"              // (package configuration)
#include "../control_flow_error.hpp"  // bsc_klammler::ControlFlowError

#include <string>                     // std::string

#define BOOST_TEST_MODULE control_flow_error
#include "module.hpp"


/**
 * @brief
 *         Unit tst for the `ControlFlowError` exception class.
 *
 * This test checks that:
 *
 * - the constructor works
 * - the `getFile` and `getLine` function members return the correct values
 * - the `what` virtual function member returns a properly formatted error
 *   message
 * - the object can be `throw`n and `catch`ed
 *
 */
BOOST_AUTO_TEST_CASE(general)
{
  class MyControlFlowError : public bsc_klammler::ControlFlowError
  {
  public:
    MyControlFlowError(const std::string& file,
                       const int line,
                       const std::string& message) :
      ControlFlowError(file, line, message)
    {
      // do nothing
    }
  };
  const std::string filename {"source.c"};
  const int linenumber {42};
  const std::string message {"so, it has come to this"};
  const std::string expected {"source.c:42: so, it has come to this"};
  const MyControlFlowError mcfe {filename, linenumber, message};
  BOOST_CHECK_EQUAL(filename, mcfe.getFile());
  BOOST_CHECK_EQUAL(linenumber, mcfe.getLine());
  BOOST_CHECK_EQUAL(expected, mcfe.what());
  try
    {
      throw mcfe;
    }
  catch (const bsc_klammler::ControlFlowError& e)
    {
      BOOST_CHECK_EQUAL(filename, e.getFile());
      BOOST_CHECK_EQUAL(linenumber, e.getLine());
      BOOST_CHECK_EQUAL(expected, e.what());
    }
}
