/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file test_not_reached.cpp
 *
 * @brief
 *         Unit tests for the `not_reached.hpp` header.
 *
 */

#include "../global.hpp"       // (package configuration)
#include "../not_reached.hpp"  // BSC_KLAMMLER_NOT_REACHED

#include <cstring>             // std::strlen
#include <string>              // std::string

#define BOOST_TEST_MODULE not_reached
#include "module.hpp"


/**
 * @brief
 *         Unit test for the `NotReachedError` exception class.
 *
 * This test checks that:
 *
 * - the class has a conforming contructor
 *
 */
BOOST_AUTO_TEST_CASE(excption)
{
  const std::string filename {"source.c"};
  const int linenumber {42};
  const std::string message {"so, it has come to this"};
  const std::string expected {"source.c:42: so, it has come to this"};
  const bsc_klammler::NotReachedError nre {filename, linenumber, message};
  BOOST_CHECK_EQUAL(filename, nre.getFile());
  BOOST_CHECK_EQUAL(linenumber, nre.getLine());
  BOOST_CHECK_EQUAL(expected, nre.what());
}


/**
 * @brief
 *         Unit test for the `BSC_KLAMMLER_NOT_REACHED` convenience macro.
 *
 * This test checks that:
 *
 * - the macro throws a correct exception
 *
 */
BOOST_AUTO_TEST_CASE(macro)
{
  const std::string expected_filename {__FILE__};
  const int expected_linenumber {__LINE__ + 3};
  try
    {
      BSC_KLAMMLER_NOT_REACHED;
      BOOST_ERROR("control flow not stopped");
    }
  catch (const bsc_klammler::NotReachedError& e)
    {
      BOOST_CHECK_EQUAL(expected_filename, e.getFile());
      BOOST_CHECK_EQUAL(expected_linenumber, e.getLine());
      BOOST_CHECK(std::strlen(e.what()) > 1);
    }
  assert(!expected_filename.empty());
  assert(expected_linenumber > 4);
}
