/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file test_utility.cpp
 *
 * @brief
 *         Unit tests for the `utility.hpp` header file.
 *
 */

#include "../global.hpp"   // (package configuration)
#include "../utility.hpp"  // (module to test)

#define BOOST_TEST_MODULE utility
#include "module.hpp"


/**
 * @brief
 *         Unit test for the `const_if` helper `struct`.
 *
 * This test checks that:
 *
 * - the `type` member type is correctly `const`-qualified
 *
 */
BOOST_AUTO_TEST_CASE(struct_const_if)
{
  using bsc_klammler::const_if;
  class Bar
  {
    int dummy;
  public:
    Bar(const int d) : dummy {d} {}
    int get() const { return this->dummy; }
    void set(const int d) { this->dummy = d; }
  };
  // The following will only compile if everything is correct.
  using cb_t = typename const_if<Bar, true>::type;
  using mb_t = typename const_if<Bar, false>::type;
  static_assert((std::is_const<cb_t>::value), "not const");
  static_assert(!(std::is_const<mb_t>::value), "not mutable");
  const auto readonly = [](const Bar& bar)->int{ return bar.get(); };
  cb_t cb {1};
  mb_t mb {2};
  mb.set(3);
  BOOST_CHECK_EQUAL(1, cb.get());
  BOOST_CHECK_EQUAL(3, mb.get());
  BOOST_CHECK_EQUAL(1, readonly(cb));
}
