/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


/**
 * @file util.hpp
 *
 * @brief
 *         Auxilliary functionality useful to implement unit tests.
 *
 */

#ifndef BSC_KLAMMLER_TEST_UTIL_HPP
#define BSC_KLAMMLER_TEST_UTIL_HPP

#include <type_traits>    // std::remove_reference

#define MY_CHECK_MATRIX_EQUAL(M1, M2)                                         \
  do                                                                          \
    {                                                                         \
      using index_type = typename std::remove_reference<decltype(M1)>::type::index_type; \
      const auto& first_ = M1;                                                \
      const auto& second_ = M2;                                               \
      BOOST_CHECK_EQUAL(first_.rows(), second_.rows());                       \
      BOOST_CHECK_EQUAL(first_.cols(), second_.cols());                       \
      if (first_.rows() ==  second_.rows() &&                                 \
          first_.cols() ==  second_.cols())                                   \
        {                                                                     \
          for (index_type j_ = 0; j_ < first_.cols(); ++j_)                   \
            {                                                                 \
              for (index_type i_ = 0; i_ < first_.rows(); ++i_)               \
                {                                                             \
                  BOOST_CHECK_EQUAL(first_.get(i_, j_), second_.get(i_, j_)); \
                }                                                             \
            }                                                                 \
        }                                                                     \
    } while (false)

#define MY_CHECK_MATRIX_CLOSE(M1, M2, Tol)                                    \
  do                                                                          \
    {                                                                         \
      using index_type = typename std::remove_reference<decltype(M1)>::type::index_type; \
      const auto& first_ = M1;                                                \
      const auto& second_ = M2;                                               \
      constexpr typename decltype(M1)::value_type tol_ {Tol};                 \
      BOOST_CHECK_EQUAL(first_.rows(), second_.rows());                       \
      BOOST_CHECK_EQUAL(first_.cols(), second_.cols());                       \
      if (first_.rows() ==  second_.rows() &&                                 \
          first_.cols() ==  second_.cols())                                   \
        {                                                                     \
          for (index_type j_ = 0; j_ < first_.cols(); ++j_)                   \
            {                                                                 \
              for (index_type i_ = 0; i_ < first_.rows(); ++i_)               \
                {                                                             \
                  BOOST_CHECK_CLOSE(first_.get(i_, j_), second_.get(i_, j_), tol_); \
                }                                                             \
            }                                                                 \
        }                                                                     \
    } while (false)


#define MY_CHECK_VECTOR_EQUAL(V1, V2)                                         \
  do                                                                          \
    {                                                                         \
      using index_type = typename std::remove_reference<decltype(V1)>::type::index_type; \
      const auto& first_ = V1;                                                \
      const auto& second_ = V2;                                               \
      BOOST_CHECK_EQUAL(first_.size(), second_.size());                       \
      if (first_.size() ==  second_.size())                                   \
        {                                                                     \
          for (index_type i_ = 0; i_ < first_.size(); ++i_)                   \
            {                                                                 \
              BOOST_CHECK_EQUAL(first_.get(i_), second_.get(i_));             \
            }                                                                 \
        }                                                                     \
    } while (false)

#define MY_CHECK_VECTOR_CLOSE(V1, V2, Tol)                                    \
  do                                                                          \
    {                                                                         \
      using index_type = typename std::remove_reference<decltype(V1)>::type::index_type; \
      const auto& first_ = V1;                                                \
      const auto& second_ = V2;                                               \
      constexpr decltype(V1)::value_type tol_ {Tol};                          \
      BOOST_CHECK_EQUAL(first_.size(), second_.size());                       \
      if (first_.size() ==  second_.size())                                   \
        {                                                                     \
          for (index_type i_ = 0; i_ < first_.size(); ++i_)                   \
            {                                                                 \
              BOOST_CHECK_CLOSE(first_.get(i_), second_.get(i_), tol_);       \
            }                                                                 \
        }                                                                     \
    } while (false)

#endif  // ifndef BSC_KLAMMLER_TEST_UTIL_HPP
